@@ -1,9 +1,11 @@
-
 import _rawffi
 from _ctypes.basics import _CData, _CDataMeta, keepalive_key,\
      store_reference, ensure_objects, CArgObject
+from _ctypes.array import Array
+from _ctypes.pointer import _Pointer
 import inspect
 
+
 def names_and_fields(self, _fields_, superclass, anonymous_fields=None):
     # _fields_: list of (name, ctype, [optional_bitfield])
     if isinstance(_fields_, tuple):
@@ -72,6 +74,7 @@ def names_and_fields(self, _fields_, superclass, anonymous_fields=None):
     for name, field in fields.items():
         setattr(self, name, field)
 
+
 class Field(object):
     def __init__(self, name, offset, size, ctype, num, is_bitfield):
         self.__dict__['name'] = name
@@ -100,12 +103,14 @@ class Field(object):
             suba = obj._subarray(fieldtype, self.name)
             return fieldtype._CData_output(suba, obj, offset)
 
-
     def __set__(self, obj, value):
         fieldtype = self.ctype
         cobj = fieldtype.from_param(value)
-        if ensure_objects(cobj) is not None:
-            key = keepalive_key(self.num)
+        key = keepalive_key(self.num)
+        if issubclass(fieldtype, _Pointer) and isinstance(cobj, Array):
+            # if our value is an Array we need the whole thing alive
+            store_reference(obj, key, cobj)
+        elif ensure_objects(cobj) is not None:
             store_reference(obj, key, cobj._objects)
         arg = cobj._get_buffer_value()
         if fieldtype._fficompositesize is not None:
@@ -116,9 +121,6 @@ class Field(object):
             obj._buffer.__setattr__(self.name, arg)
 
 
-
-# ________________________________________________________________
-
 def _set_shape(tp, rawfields, is_union=False):
     tp._ffistruct = _rawffi.Structure(rawfields, is_union,
                                       getattr(tp, '_pack_', 0))
@@ -140,8 +142,8 @@ def struct_setattr(self, name, value):
         return
     _CDataMeta.__setattr__(self, name, value)
 
-class StructOrUnionMeta(_CDataMeta):
 
+class StructOrUnionMeta(_CDataMeta):
     def __new__(self, name, cls, typedict):
         res = type.__new__(self, name, cls, typedict)
         if "_abstract_" in typedict:
@@ -149,6 +151,9 @@ class StructOrUnionMeta(_CDataMeta):
         cls = cls or (object,)
         if isinstance(cls[0], StructOrUnionMeta):
             cls[0]._make_final()
+        if '_pack_' in typedict:
+            if not 0 <= typedict['_pack_'] < 2**31:
+                raise ValueError("_pack_ must be a non-negative integer")
         if '_fields_' in typedict:
             if not hasattr(typedict.get('_anonymous_', []), '__iter__'):
                 raise TypeError("Anonymous field must be iterable")
@@ -159,7 +164,6 @@ class StructOrUnionMeta(_CDataMeta):
                 res,
                 typedict['_fields_'], cls[0],
                 typedict.get('_anonymous_', None))
-
         return res
 
     def _make_final(self):
@@ -201,7 +205,7 @@ class StructOrUnionMeta(_CDataMeta):
         res.__dict__['_base'] = base
         res.__dict__['_index'] = index
         return res
-    
+
     def _CData_retval(self, resbuffer):
         res = StructOrUnion.__new__(self)
         res.__dict__['_buffer'] = resbuffer
@@ -209,6 +213,7 @@ class StructOrUnionMeta(_CDataMeta):
         res.__dict__['_index'] = -1
         return res
 
+
 class StructOrUnion(_CData):
     __metaclass__ = StructOrUnionMeta
 
@@ -252,5 +257,6 @@ class StructOrUnion(_CData):
 class StructureMeta(StructOrUnionMeta):
     _is_union = False
 
+
 class Structure(StructOrUnion):
     __metaclass__ = StructureMeta
