@@ -12,9 +12,26 @@ class partial(object):
     def __init__(self, func, *args, **keywords):
         if not callable(func):
             raise TypeError("the first argument must be callable")
-        self.func = func
-        self.args = args
-        self.keywords = keywords or None
+        self._func = func
+        self._args = args
+        self._keywords = keywords or None
+
+    def __delattr__(self, key):
+        if key == '__dict__':
+            raise TypeError("a partial object's dictionary may not be deleted")
+        object.__delattr__(self, key)
+
+    @property
+    def func(self):
+        return self._func
+
+    @property
+    def args(self):
+        return self._args
+
+    @property
+    def keywords(self):
+        return self._keywords
 
     def __call__(self, *fargs, **fkeywords):
         if self.keywords is not None:
@@ -23,13 +40,13 @@ class partial(object):
 
     def __reduce__(self):
         d = dict((k, v) for k, v in self.__dict__.iteritems() if k not in
-                ('func', 'args', 'keywords'))
+                ('_func', '_args', '_keywords'))
         if len(d) == 0:
             d = None
         return (type(self), (self.func,),
                 (self.func, self.args, self.keywords, d))
 
     def __setstate__(self, state):
-        self.func, self.args, self.keywords, d = state
+        self._func, self._args, self._keywords, d = state
         if d is not None:
             self.__dict__.update(d)
