@@ -1,21 +1,28 @@
 import py
-import sys, os.path
+import sys
+from inspect import CO_VARARGS, CO_VARKEYWORDS
 
 builtin_repr = repr
 
 reprlib = py.builtin._tryimport('repr', 'reprlib')
 
+if sys.version_info[0] >= 3:
+    from traceback import format_exception_only
+else:
+    from py._code._py2traceback import format_exception_only
+
 class Code(object):
     """ wrapper around Python code objects """
     def __init__(self, rawcode):
-        rawcode = py.code.getrawcode(rawcode)
-        self.raw = rawcode
+        if not hasattr(rawcode, "co_filename"):
+            rawcode = py.code.getrawcode(rawcode)
         try:
             self.filename = rawcode.co_filename
             self.firstlineno = rawcode.co_firstlineno - 1
             self.name = rawcode.co_name
         except AttributeError:
             raise TypeError("not a code object: %r" %(rawcode,))
+        self.raw = rawcode
 
     def __eq__(self, other):
         return self.raw == other.raw
@@ -23,25 +30,25 @@ class Code(object):
     def __ne__(self, other):
         return not self == other
 
+    @property
     def path(self):
-        """ return a path object pointing to source code"""
+        """ return a path object pointing to source code (note that it
+        might not point to an actually existing file). """
         p = py.path.local(self.raw.co_filename)
+        # maybe don't try this checking
         if not p.check():
             # XXX maybe try harder like the weird logic
             # in the standard lib [linecache.updatecache] does?
             p = self.raw.co_filename
         return p
 
-    path = property(path, None, None, "path of this code object")
-
+    @property
     def fullsource(self):
         """ return a py.code.Source object for the full source file of the code
         """
         from py._code import source
         full, _ = source.findsource(self.raw)
         return full
-    fullsource = property(fullsource, None, None,
-                          "full source containing this code object")
 
     def source(self):
         """ return a py.code.Source object for the code object's source only
@@ -49,30 +56,37 @@ class Code(object):
         # return source only for that part of code
         return py.code.Source(self.raw)
 
-    def getargs(self):
+    def getargs(self, var=False):
         """ return a tuple with the argument names for the code object
+
+            if 'var' is set True also return the names of the variable and
+            keyword arguments when present
         """
         # handfull shortcut for getting args
         raw = self.raw
-        return raw.co_varnames[:raw.co_argcount]
+        argcount = raw.co_argcount
+        if var:
+            argcount += raw.co_flags & CO_VARARGS
+            argcount += raw.co_flags & CO_VARKEYWORDS
+        return raw.co_varnames[:argcount]
 
 class Frame(object):
     """Wrapper around a Python frame holding f_locals and f_globals
     in which expressions can be evaluated."""
 
     def __init__(self, frame):
-        self.code = py.code.Code(frame.f_code)
         self.lineno = frame.f_lineno - 1
         self.f_globals = frame.f_globals
         self.f_locals = frame.f_locals
         self.raw = frame
+        self.code = py.code.Code(frame.f_code)
 
+    @property
     def statement(self):
+        """ statement this frame is at """
         if self.code.fullsource is None:
             return py.code.Source("")
         return self.code.fullsource.getstatement(self.lineno)
-    statement = property(statement, None, None,
-                         "statement this frame is at")
 
     def eval(self, code, **vars):
         """ evaluate 'code' in the frame
@@ -102,11 +116,14 @@ class Frame(object):
     def is_true(self, object):
         return object
 
-    def getargs(self):
+    def getargs(self, var=False):
         """ return a list of tuples (name, value) for all arguments
+
+            if 'var' is set True also include the variable and keyword
+            arguments when present
         """
         retval = []
-        for arg in self.code.getargs():
+        for arg in self.code.getargs(var):
             try:
                 retval.append((arg, self.f_locals[arg]))
             except KeyError:
@@ -120,26 +137,29 @@ class TracebackEntry(object):
 
     def __init__(self, rawentry):
         self._rawentry = rawentry
-        self.frame = py.code.Frame(rawentry.tb_frame)
-        # Ugh. 2.4 and 2.5 differs here when encountering
-        # multi-line statements. Not sure about the solution, but
-        # should be portable
         self.lineno = rawentry.tb_lineno - 1
-        self.relline = self.lineno - self.frame.code.firstlineno
+
+    @property
+    def frame(self):
+        return py.code.Frame(self._rawentry.tb_frame)
+
+    @property
+    def relline(self):
+        return self.lineno - self.frame.code.firstlineno
 
     def __repr__(self):
         return "<TracebackEntry %s:%d>" %(self.frame.code.path, self.lineno+1)
 
+    @property
     def statement(self):
-        """ return a py.code.Source object for the current statement """
+        """ py.code.Source object for the current statement """
         source = self.frame.code.fullsource
         return source.getstatement(self.lineno)
-    statement = property(statement, None, None,
-                         "statement of this traceback entry.")
 
+    @property
     def path(self):
+        """ path to the source code """
         return self.frame.code.path
-    path = property(path, None, None, "path to the full source code")
 
     def getlocals(self):
         return self.frame.f_locals
@@ -160,26 +180,30 @@ class TracebackEntry(object):
         # on Jython this firstlineno can be -1 apparently
         return max(self.frame.code.firstlineno, 0)
 
-    def getsource(self):
+    def getsource(self, astcache=None):
         """ return failing source code. """
+        # we use the passed in astcache to not reparse asttrees
+        # within exception info printing
+        from py._code.source import getstatementrange_ast
         source = self.frame.code.fullsource
         if source is None:
             return None
+        key = astnode = None
+        if astcache is not None:
+            key = self.frame.code.path
+            if key is not None:
+                astnode = astcache.get(key, None)
         start = self.getfirstlinesource()
-        end = self.lineno
         try:
-            _, end = source.getstatementrange(end)
-        except (IndexError, ValueError):
+            astnode, _, end = getstatementrange_ast(self.lineno, source,
+                                                    astnode=astnode)
+        except SyntaxError:
             end = self.lineno + 1
-        # heuristic to stop displaying source on e.g.
-        #   if something:  # assume this causes a NameError
-        #      # _this_ lines and the one
-               #        below we don't want from entry.getsource()
-        for i in range(self.lineno, end):
-            if source[i].rstrip().endswith(':'):
-                end = i + 1
-                break
+        else:
+            if key is not None:
+                astcache[key] = astnode
         return source[start:end]
+
     source = property(getsource)
 
     def ishidden(self):
@@ -189,11 +213,12 @@ class TracebackEntry(object):
             mostly for internal use
         """
         try:
-            return self.frame.eval("__tracebackhide__")
-        except py.builtin._sysex:
-            raise
-        except:
-            return False
+            return self.frame.f_locals['__tracebackhide__']
+        except KeyError:
+            try:
+                return self.frame.f_globals['__tracebackhide__']
+            except KeyError:
+                return False
 
     def __str__(self):
         try:
@@ -272,10 +297,11 @@ class Traceback(list):
         """ return last non-hidden traceback entry that lead
         to the exception of a traceback.
         """
-        tb = self.filter()
-        if not tb:
-            tb = self
-        return tb[-1]
+        for i in range(-1, -len(self)-1, -1):
+            entry = self[i]
+            if not entry.ishidden():
+                return entry
+        return self[-1]
 
     def recursionindex(self):
         """ return the index of the frame/TracebackItem where recursion
@@ -310,8 +336,6 @@ class ExceptionInfo(object):
     """
     _striptext = ''
     def __init__(self, tup=None, exprinfo=None):
-        # NB. all attributes are private!  Subclasses or other
-        #     ExceptionInfo-like classes may have different attributes.
         if tup is None:
             tup = sys.exc_info()
             if exprinfo is None and isinstance(tup[1], AssertionError):
@@ -321,9 +345,16 @@ class ExceptionInfo(object):
                 if exprinfo and exprinfo.startswith('assert '):
                     self._striptext = 'AssertionError: '
         self._excinfo = tup
-        self.type, self.value, tb = self._excinfo
+        #: the exception class
+        self.type = tup[0]
+        #: the exception instance
+        self.value = tup[1]
+        #: the exception raw traceback
+        self.tb = tup[2]
+        #: the exception type name
         self.typename = self.type.__name__
-        self.traceback = py.code.Traceback(tb)
+        #: the exception traceback (py.code.Traceback instance)
+        self.traceback = py.code.Traceback(self.tb)
 
     def __repr__(self):
         return "<ExceptionInfo %s tblen=%d>" % (self.typename, len(self.traceback))
@@ -336,7 +367,7 @@ class ExceptionInfo(object):
             the exception representation is returned (so 'AssertionError: ' is
             removed from the beginning)
         """
-        lines = py.std.traceback.format_exception_only(self.type, self.value)
+        lines = format_exception_only(self.type, self.value)
         text = ''.join(lines)
         text = text.rstrip()
         if tryshort:
@@ -351,9 +382,8 @@ class ExceptionInfo(object):
     def _getreprcrash(self):
         exconly = self.exconly(tryshort=True)
         entry = self.traceback.getcrashentry()
-        path, lineno = entry.path, entry.lineno
-        reprcrash = ReprFileLocation(path, lineno+1, exconly)
-        return reprcrash
+        path, lineno = entry.frame.code.raw.co_filename, entry.lineno
+        return ReprFileLocation(path, lineno+1, exconly)
 
     def getrepr(self, showlocals=False, style="long",
             abspath=False, tbfilter=True, funcargs=False):
@@ -399,6 +429,7 @@ class FormattedExcinfo(object):
         self.tbfilter = tbfilter
         self.funcargs = funcargs
         self.abspath = abspath
+        self.astcache = {}
 
     def _getindent(self, source):
         # figure out indent for given source
@@ -416,7 +447,7 @@ class FormattedExcinfo(object):
         return 4 + (len(s) - len(s.lstrip()))
 
     def _getentrysource(self, entry):
-        source = entry.getsource()
+        source = entry.getsource(self.astcache)
         if source is not None:
             source = source.deindent()
         return source
@@ -427,7 +458,7 @@ class FormattedExcinfo(object):
     def repr_args(self, entry):
         if self.funcargs:
             args = []
-            for argname, argvalue in entry.frame.getargs():
+            for argname, argvalue in entry.frame.getargs(var=True):
                 args.append((argname, self._saferepr(argvalue)))
             return ReprFuncArgs(args)
 
@@ -557,27 +588,21 @@ class FormattedExcinfo(object):
 class TerminalRepr:
     def __str__(self):
         s = self.__unicode__()
-        if sys.version_info[0] < 3:
+        if sys.version_info[0] < 3 and isinstance(s, unicode):
             s = s.encode('utf-8')
         return s
 
     def __unicode__(self):
-        l = []
-        tw = py.io.TerminalWriter(l.append)
+        # FYI this is called from pytest-xdist's serialization of exception
+        # information.
+        io = py.io.TextIO()
+        tw = py.io.TerminalWriter(file=io)
         self.toterminal(tw)
-        l = map(unicode_or_repr, l)
-        return "".join(l).strip()
+        return io.getvalue().strip()
 
     def __repr__(self):
         return "<%s instance at %0x>" %(self.__class__, id(self))
 
-def unicode_or_repr(obj):
-    try:
-        return py.builtin._totext(obj)
-    except KeyboardInterrupt:
-        raise
-    except Exception:
-        return "<print-error: %r>" % py.io.saferepr(obj)
 
 class ReprExceptionInfo(TerminalRepr):
     def __init__(self, reprtraceback, reprcrash):
