@@ -93,6 +93,7 @@ class DummySpace(object):
 
     def setitem(self, obj, key, value):
         obj[key] = value
+    setitem_str = setitem
 
     def getitem(self, obj, key):
         return obj[key]
@@ -106,7 +107,7 @@ class DummySpace(object):
     def len(self, x):
         return len(x)
 
-    def int_w(self, x):
+    def int_w(self, x, allow_conversion=True):
         return x
 
     def eq_w(self, x, y):
