@@ -167,6 +167,40 @@ class TestObjSpace:
         self.space.setattr(w_oldstyle, self.space.wrap("__call__"), w_func)
         assert is_callable(w_oldstyle)
 
+    def test_int_w(self):
+        space = self.space
+        w_x = space.wrap(42)
+        assert space.int_w(w_x) == 42
+        assert space.int_w(w_x, allow_conversion=False) == 42
+        #
+        w_x = space.wrap(44.0)
+        space.raises_w(space.w_TypeError, space.int_w, w_x)
+        space.raises_w(space.w_TypeError, space.int_w, w_x, allow_conversion=False)
+        #
+        w_instance = self.space.appexec([], """():
+            class MyInt(object):
+                def __int__(self):
+                    return 43
+            return MyInt()
+        """)
+        assert space.int_w(w_instance) == 43
+        space.raises_w(space.w_TypeError, space.int_w, w_instance, allow_conversion=False)
+        #
+        w_instance = self.space.appexec([], """():
+            class MyInt(object):
+                def __int__(self):
+                    return 43
+
+            class AnotherInt(object):
+                def __int__(self):
+                    return MyInt()
+
+            return AnotherInt()
+        """)
+        space.raises_w(space.w_TypeError, space.int_w, w_instance)
+        space.raises_w(space.w_TypeError, space.int_w, w_instance, allow_conversion=False)
+
+
     def test_interp_w(self):
         w = self.space.wrap
         w_bltinfunction = self.space.builtin.get('len')
