@@ -31,8 +31,10 @@ class AppTestBuiltinApp:
         emptyfile.write('')
         nullbytes = udir.join('nullbytes.py')
         nullbytes.write('#abc\x00def\n')
+        nonexistent = udir.join('builtins-nonexistent')
         cls.w_emptyfile = space.wrap(str(emptyfile))
         cls.w_nullbytes = space.wrap(str(nullbytes))
+        cls.w_nonexistent = space.wrap(str(nonexistent))
 
     def test_builtin_names(self):
         import __builtin__
@@ -627,6 +629,9 @@ def fn(): pass
         raises(TypeError, compile, src, 'mymod', 'exec', 0)
         execfile(self.nullbytes) # works
 
+    def test_execfile_args(self):
+        raises(TypeError, execfile, self.nonexistent, {}, ())
+
     def test_compile_null_bytes_flag(self):
         try:
             from _ast import PyCF_ACCEPT_NULL_BYTES
