@@ -1,11 +1,11 @@
 import operator
 
 from pypy.interpreter.baseobjspace import W_Root
-from pypy.interpreter.error import OperationError, operationerrfmt
+from pypy.interpreter.error import OperationError, oefmt
 from pypy.interpreter.gateway import interp2app
 from pypy.interpreter.typedef import TypeDef, make_weakref_descr
 
-from rpython.rlib import objectmodel, rgc
+from rpython.rlib import rgc
 from rpython.rlib.objectmodel import keepalive_until_here, specialize
 from rpython.rtyper.lltypesystem import lltype, rffi
 from rpython.tool.sourcetools import func_with_new_name
@@ -78,9 +78,8 @@ class W_CData(W_Root):
         space = self.space
         if isinstance(self.ctype, ctypearray.W_CTypeArray):
             return space.wrap(self.get_array_length())
-        raise operationerrfmt(space.w_TypeError,
-                              "cdata of type '%s' has no len()",
-                              self.ctype.name)
+        raise oefmt(space.w_TypeError,
+                    "cdata of type '%s' has no len()", self.ctype.name)
 
     def _make_comparison(name):
         op = getattr(operator, name)
@@ -206,8 +205,7 @@ class W_CData(W_Root):
                 w_value.get_array_length() == length):
                 # fast path: copying from exactly the correct type
                 s = w_value._cdata
-                for i in range(ctitemsize * length):
-                    cdata[i] = s[i]
+                rffi.c_memcpy(cdata, s, ctitemsize * length)
                 keepalive_until_here(w_value)
                 return
         #
@@ -220,9 +218,9 @@ class W_CData(W_Root):
             from rpython.rtyper.lltypesystem.rstr import copy_string_to_raw
             value = space.str_w(w_value)
             if len(value) != length:
-                raise operationerrfmt(space.w_ValueError,
-                                      "need a string of length %d, got %d",
-                                      length, len(value))
+                raise oefmt(space.w_ValueError,
+                            "need a string of length %d, got %d",
+                            length, len(value))
             copy_string_to_raw(llstr(value), cdata, 0, length)
             return
         #
@@ -233,9 +231,8 @@ class W_CData(W_Root):
             except OperationError, e:
                 if not e.match(space, space.w_StopIteration):
                     raise
-                raise operationerrfmt(space.w_ValueError,
-                                      "need %d values to unpack, got %d",
-                                      length, i)
+                raise oefmt(space.w_ValueError,
+                            "need %d values to unpack, got %d", length, i)
             ctitem.convert_from_object(cdata, w_item)
             cdata = rffi.ptradd(cdata, ctitemsize)
         try:
@@ -244,8 +241,8 @@ class W_CData(W_Root):
             if not e.match(space, space.w_StopIteration):
                 raise
         else:
-            raise operationerrfmt(space.w_ValueError,
-                                  "got more than %d values to unpack", length)
+            raise oefmt(space.w_ValueError,
+                        "got more than %d values to unpack", length)
 
     def _add_or_sub(self, w_other, sign):
         space = self.space
@@ -259,7 +256,6 @@ class W_CData(W_Root):
         space = self.space
         if isinstance(w_other, W_CData):
             from pypy.module._cffi_backend import ctypeptr, ctypearray
-            from pypy.module._cffi_backend import ctypevoid
             ct = w_other.ctype
             if isinstance(ct, ctypearray.W_CTypeArray):
                 ct = ct.ctptr
@@ -267,9 +263,9 @@ class W_CData(W_Root):
             if (ct is not self.ctype or
                    not isinstance(ct, ctypeptr.W_CTypePointer) or
                    (ct.ctitem.size <= 0 and not ct.is_void_ptr)):
-                raise operationerrfmt(space.w_TypeError,
-                    "cannot subtract cdata '%s' and cdata '%s'",
-                    self.ctype.name, ct.name)
+                raise oefmt(space.w_TypeError,
+                            "cannot subtract cdata '%s' and cdata '%s'",
+                            self.ctype.name, ct.name)
             #
             itemsize = ct.ctitem.size
             if itemsize <= 0: itemsize = 1
@@ -424,6 +420,12 @@ class W_CDataSliced(W_CData):
     def get_array_length(self):
         return self.length
 
+    def _sizeof(self):
+        from pypy.module._cffi_backend.ctypeptr import W_CTypePtrOrArray
+        ctype = self.ctype
+        assert isinstance(ctype, W_CTypePtrOrArray)
+        return self.length * ctype.ctitem.size
+
 
 class W_CDataHandle(W_CData):
     _attrs_ = ['w_keepalive']
@@ -439,7 +441,7 @@ class W_CDataHandle(W_CData):
 
 
 W_CData.typedef = TypeDef(
-    'CData',
+    '_cffi_backend.CData',
     __module__ = '_cffi_backend',
     __name__ = '<cdata>',
     __repr__ = interp2app(W_CData.repr),
