@@ -4,7 +4,7 @@ from rpython.rlib import streamio
 from rpython.rlib.streamio import StreamErrors
 
 from pypy.interpreter.error import OperationError
-from pypy.interpreter.baseobjspace import ObjSpace, W_Root
+from pypy.interpreter.baseobjspace import ObjSpace, W_Root, CannotHaveLock
 from pypy.interpreter.typedef import TypeDef
 from pypy.interpreter.gateway import interp2app
 from pypy.interpreter.streamutil import wrap_streamerror, wrap_oserror_as_ioerror
@@ -33,19 +33,24 @@ class W_AbstractStream(W_Root):
     def _try_acquire_lock(self):
         # this function runs with the GIL acquired so there is no race
         # condition in the creation of the lock
-        if self.slock is None:
-            self.slock = self.space.allocate_lock()
         me = self.space.getexecutioncontext()   # used as thread ident
         if self.slockowner is me:
             return False    # already acquired by the current thread
-        self.slock.acquire(True)
+        try:
+            if self.slock is None:
+                self.slock = self.space.allocate_lock()
+        except CannotHaveLock:
+            pass
+        else:
+            self.slock.acquire(True)
         assert self.slockowner is None
         self.slockowner = me
         return True
 
     def _release_lock(self):
         self.slockowner = None
-        self.slock.release()
+        if self.slock is not None:
+            self.slock.release()
 
     def lock(self):
         if not self._try_acquire_lock():
