@@ -919,9 +919,15 @@ def make_forwarding_method(method, writer=False, reader=False):
     @func_renamer(method + '_w')
     def method_w(self, space, __args__):
         if writer:
+            if self.w_writer is None:
+                raise oefmt(space.w_ValueError,
+                            "I/O operation on uninitialized object")
             w_meth = space.getattr(self.w_writer, space.wrap(method))
             w_result = space.call_args(w_meth, __args__)
         if reader:
+            if self.w_reader is None:
+                raise oefmt(space.w_ValueError,
+                            "I/O operation on uninitialized object")
             w_meth = space.getattr(self.w_reader, space.wrap(method))
             w_result = space.call_args(w_meth, __args__)
         return w_result
