@@ -139,6 +139,14 @@ class AppTestBufferedReader:
         raw = _io.FileIO(self.tmpfile)
         f = _io.BufferedReader(raw)
         assert f.readinto(a) == 5
+        exc = raises(TypeError, f.readinto, u"hello")
+        assert str(exc.value) == "cannot use unicode as modifiable buffer"
+        exc = raises(TypeError, f.readinto, buffer(b"hello"))
+        assert str(exc.value) == "must be read-write buffer, not buffer"
+        exc = raises(TypeError, f.readinto, buffer(bytearray("hello")))
+        assert str(exc.value) == "must be read-write buffer, not buffer"
+        exc = raises(TypeError, f.readinto, memoryview(b"hello"))
+        assert str(exc.value) == "must be read-write buffer, not memoryview"
         f.close()
         assert a == 'a\nb\ncxxxxx'
 
@@ -235,7 +243,8 @@ class AppTestBufferedWriter:
         import _io
         raw = _io.FileIO(self.tmpfile, 'w')
         f = _io.BufferedWriter(raw)
-        f.write("abcd")
+        f.write("ab")
+        f.write(u"cd")
         f.close()
         assert self.readfile() == "abcd"
 
@@ -513,6 +522,35 @@ class AppTestBufferedWriter:
         exc = raises(RuntimeError, bufio.flush)
         assert "reentrant" in str(exc.value)  # And not e.g. recursion limit.
 
+    def test_write_error_on_close(self):
+        import _io
+        class MockRawIO(_io._RawIOBase):
+            def writable(self):
+                return True
+            def write(self, data):
+                raise IOError()
+        raw = MockRawIO()
+        b = _io.BufferedWriter(raw)
+        b.write(b'spam')
+        raises(IOError, b.close)  # exception not swallowed
+        assert b.closed
+
+    def test_close_error_on_close(self):
+        import _io
+        class MockRawIO(_io._RawIOBase):
+            def writable(self):
+                return True
+            def close(self):
+                raise IOError('close')
+        def bad_flush():
+            raise IOError('flush')
+        raw = MockRawIO()
+        b = _io.BufferedWriter(raw)
+        b.flush = bad_flush
+        err = raises(IOError, b.close)  # exception not swallowed
+        assert err.value.args == ('close',)
+        assert not b.closed
+
 class AppTestBufferedRWPair:
     def test_pair(self):
         import _io
