@@ -1,6 +1,7 @@
 from rpython.tool.udir import udir
 import os
 
+
 class AppTestFileIO:
     spaceconfig = dict(usemodules=['_io'] + (['fcntl'] if os.name != 'nt' else []))
 
@@ -17,10 +18,16 @@ class AppTestFileIO:
         import _io
         f = _io.FileIO(self.tmpfile, 'a')
         assert f.name.endswith('tmpfile')
-        assert f.mode == 'wb'
+        assert f.mode == 'ab'
         assert f.closefd is True
         f.close()
 
+    def test_invalid_fd(self):
+        import _io
+        raises(ValueError, _io.FileIO, -10)
+        raises(TypeError, _io.FileIO, 2 ** 31)
+        raises(TypeError, _io.FileIO, -2 ** 31 - 1)
+
     def test_weakrefable(self):
         import _io
         from weakref import proxy
@@ -75,7 +82,8 @@ class AppTestFileIO:
         import _io
         filename = self.tmpfile + '_w'
         f = _io.FileIO(filename, 'wb')
-        f.write("test")
+        f.write("te")
+        f.write(u"st")
         # try without flushing
         f2 = _io.FileIO(filename, 'rb')
         assert f2.read() == "test"
@@ -128,6 +136,14 @@ class AppTestFileIO:
         a = bytearray('x' * 10)
         f = _io.FileIO(self.tmpfile, 'r+')
         assert f.readinto(a) == 10
+        exc = raises(TypeError, f.readinto, u"hello")
+        assert str(exc.value) == "cannot use unicode as modifiable buffer"
+        exc = raises(TypeError, f.readinto, buffer(b"hello"))
+        assert str(exc.value) == "must be read-write buffer, not buffer"
+        exc = raises(TypeError, f.readinto, buffer(bytearray("hello")))
+        assert str(exc.value) == "must be read-write buffer, not buffer"
+        exc = raises(TypeError, f.readinto, memoryview(b"hello"))
+        assert str(exc.value) == "must be read-write buffer, not memoryview"
         f.close()
         assert a == 'a\nb\nc\0\0\0\0\0'
         #
@@ -165,6 +181,35 @@ class AppTestFileIO:
         f.close()
         assert repr(f) == "<_io.FileIO [closed]>"
 
+    def test_unclosed_fd_on_exception(self):
+        import _io
+        import os
+        class MyException(Exception): pass
+        class MyFileIO(_io.FileIO):
+            def __setattr__(self, name, value):
+                if name == "name":
+                    raise MyException("blocked setting name")
+                return super(MyFileIO, self).__setattr__(name, value)
+        fd = os.open(self.tmpfile, os.O_RDONLY)
+        raises(MyException, MyFileIO, fd)
+        os.close(fd)  # should not raise OSError(EBADF)
+
+    def test_mode_strings(self):
+        import _io
+        import os
+        try:
+            for modes in [('w', 'wb'), ('wb', 'wb'), ('wb+', 'rb+'),
+                          ('w+b', 'rb+'), ('a', 'ab'), ('ab', 'ab'),
+                          ('ab+', 'ab+'), ('a+b', 'ab+'), ('r', 'rb'),
+                          ('rb', 'rb'), ('rb+', 'rb+'), ('r+b', 'rb+')]:
+                # read modes are last so that TESTFN will exist first
+                with _io.FileIO(self.tmpfile, modes[0]) as f:
+                    assert f.mode == modes[1]
+        finally:
+            if os.path.exists(self.tmpfile):
+                os.unlink(self.tmpfile)
+
+
 def test_flush_at_exit():
     from pypy import conftest
     from pypy.tool.option import make_config, make_objspace
@@ -176,13 +221,14 @@ def test_flush_at_exit():
     space.appexec([space.wrap(str(tmpfile))], """(tmpfile):
         import io
         f = io.open(tmpfile, 'w', encoding='ascii')
-        f.write('42')
+        f.write(u'42')
         # no flush() and no close()
         import sys; sys._keepalivesomewhereobscure = f
     """)
     space.finish()
     assert tmpfile.read() == '42'
 
+
 def test_flush_at_exit_IOError_and_ValueError():
     from pypy import conftest
     from pypy.tool.option import make_config, make_objspace
