@@ -5,7 +5,7 @@ Tests for the md5 module implemented at interp-level in pypy/module/md5.
 
 class AppTestMD5(object):
     spaceconfig = {
-        "usemodules": ['_md5', 'rctime', 'binascii'],
+        'usemodules': ['_md5', 'binascii', 'rctime', 'struct'],
     }
 
     def setup_class(cls):
