@@ -1,4 +1,3 @@
-
 """ Interpreter-level implementation of structure, exposing ll-structure
 to app-level with apropriate interface
 """
@@ -6,8 +5,7 @@ to app-level with apropriate interface
 from pypy.interpreter.gateway import interp2app, unwrap_spec
 from pypy.interpreter.typedef import interp_attrproperty
 from pypy.interpreter.typedef import TypeDef, GetSetProperty
-from rpython.rtyper.lltypesystem import lltype, rffi
-from pypy.interpreter.error import OperationError, operationerrfmt
+from pypy.interpreter.error import OperationError, oefmt
 from pypy.module._rawffi.interp_rawffi import segfault_exception, _MS_WINDOWS
 from pypy.module._rawffi.interp_rawffi import W_DataShape, W_DataInstance
 from pypy.module._rawffi.interp_rawffi import wrap_value, unwrap_value
@@ -16,9 +14,12 @@ from pypy.module._rawffi.interp_rawffi import LL_TYPEMAP
 from pypy.module._rawffi.interp_rawffi import unroll_letters_for_numbers
 from pypy.module._rawffi.interp_rawffi import size_alignment
 from pypy.module._rawffi.interp_rawffi import read_ptr, write_ptr
-from rpython.rlib import clibffi
-from rpython.rlib.rarithmetic import intmask, signedtype, widen
-from rpython.rlib.rarithmetic import r_uint, r_ulonglong, r_longlong
+from rpython.rlib import clibffi, rgc
+from rpython.rlib.rarithmetic import intmask, signedtype, r_uint, \
+    r_ulonglong
+from rpython.rtyper.lltypesystem import lltype, rffi
+
+
 
 def unpack_fields(space, w_fields):
     fields_w = space.unpackiterable(w_fields)
@@ -34,8 +35,8 @@ def unpack_fields(space, w_fields):
         try:
             name = space.str_w(l_w[0])
         except OperationError:
-            raise operationerrfmt(space.w_TypeError,
-                "structure field name must be string not %T", l_w[0])
+            raise oefmt(space.w_TypeError,
+                        "structure field name must be string not %T", l_w[0])
         tp = unpack_shape_with_length(space, l_w[1])
 
         if len_l == 3:
@@ -144,8 +145,8 @@ class W_Structure(W_DataShape):
             for i in range(len(fields)):
                 name, tp, bitsize = fields[i]
                 if name in name_to_index:
-                    raise operationerrfmt(space.w_ValueError,
-                        "duplicate field name %s", name)
+                    raise oefmt(space.w_ValueError,
+                                "duplicate field name %s", name)
                 name_to_index[name] = i
             size, alignment, pos, bitsizes = size_alignment_pos(
                 fields, is_union, pack)
@@ -170,8 +171,8 @@ class W_Structure(W_DataShape):
         try:
             return self.name_to_index[attr]
         except KeyError:
-            raise operationerrfmt(space.w_AttributeError,
-                "C Structure has no attribute %s", attr)
+            raise oefmt(space.w_AttributeError,
+                        "C Structure has no attribute %s", attr)
 
     @unwrap_spec(autofree=bool)
     def descr_call(self, space, autofree=False):
@@ -226,6 +227,7 @@ class W_Structure(W_DataShape):
                                                            fieldtypes)
         return self.ffi_struct.ffistruct
 
+    @rgc.must_be_light_finalizer
     def __del__(self):
         if self.ffi_struct:
             lltype.free(self.ffi_struct, flavor='raw')
@@ -268,31 +270,30 @@ def NUM_BITS(x):
     return x >> 16
 
 def BIT_MASK(x, ll_t):
-    if ll_t is lltype.SignedLongLong:
-        return (r_longlong(1) << x) - 1
-    elif ll_t is lltype.UnsignedLongLong:
-        return (r_ulonglong(1) << x) - 1
-    return (1  << x) -1
+    if ll_t is lltype.SignedLongLong or ll_t is lltype.UnsignedLongLong:
+        one = r_ulonglong(1)
+    else:
+        one = r_uint(1)
+    # to avoid left shift by x == sizeof(ll_t)
+    return (((one << (x - 1)) - 1) << 1) + 1
 BIT_MASK._annspecialcase_ = 'specialize:arg(1)'
 
 def push_field(self, num, value):
     ptr = rffi.ptradd(self.ll_buffer, self.shape.ll_positions[num])
     TP = lltype.typeOf(value)
-    T = lltype.Ptr(rffi.CArray(TP))
-
     # Handle bitfields
     for c in unroll_letters_for_numbers:
         if LL_TYPEMAP[c] is TP and self.shape.ll_bitsizes:
             # Modify the current value with the bitfield changed
             bitsize = self.shape.ll_bitsizes[num]
             numbits = NUM_BITS(bitsize)
-            lowbit = LOW_BIT(bitsize)
             if numbits:
-                value = widen(value)
+                lowbit = LOW_BIT(bitsize)
                 bitmask = BIT_MASK(numbits, TP)
-                #
-                current = widen(read_ptr(ptr, 0, TP))
-                current &= ~ (bitmask << lowbit)
+                masktype = lltype.typeOf(bitmask)
+                value = rffi.cast(masktype, value)
+                current = rffi.cast(masktype, read_ptr(ptr, 0, TP))
+                current &= ~(bitmask << lowbit)
                 current |= (value & bitmask) << lowbit
                 value = rffi.cast(TP, current)
             break
@@ -301,29 +302,25 @@ push_field._annspecialcase_ = 'specialize:argtype(2)'
 
 def cast_pos(self, i, ll_t):
     pos = rffi.ptradd(self.ll_buffer, self.shape.ll_positions[i])
-    TP = lltype.Ptr(rffi.CArray(ll_t))
     value = read_ptr(pos, 0, ll_t)
-
     # Handle bitfields
     for c in unroll_letters_for_numbers:
         if LL_TYPEMAP[c] is ll_t and self.shape.ll_bitsizes:
             bitsize = self.shape.ll_bitsizes[i]
             numbits = NUM_BITS(bitsize)
-            lowbit = LOW_BIT(bitsize)
             if numbits:
-                value = widen(rffi.cast(ll_t, value))
+                lowbit = LOW_BIT(bitsize)
                 bitmask = BIT_MASK(numbits, ll_t)
-                #
+                masktype = lltype.typeOf(bitmask)
+                value = rffi.cast(masktype, value)
                 value >>= lowbit
                 value &= bitmask
                 if ll_t is lltype.Bool or signedtype(ll_t._type):
                     sign = (value >> (numbits - 1)) & 1
                     if sign:
-                        one = r_longlong(1) if ll_t is lltype.SignedLongLong else 1
-                        value = value - (one << numbits)
+                        value -= bitmask + 1
                 value = rffi.cast(ll_t, value)
             break
-
     return value
 cast_pos._annspecialcase_ = 'specialize:arg(2)'
 
@@ -367,7 +364,6 @@ W_StructureInstance.typedef = TypeDef(
     __repr__    = interp2app(W_StructureInstance.descr_repr),
     __getattr__ = interp2app(W_StructureInstance.getattr),
     __setattr__ = interp2app(W_StructureInstance.setattr),
-    __buffer__  = interp2app(W_StructureInstance.descr_buffer),
     buffer      = GetSetProperty(W_StructureInstance.getbuffer),
     free        = interp2app(W_StructureInstance.free),
     shape       = interp_attrproperty('shape', W_StructureInstance),
@@ -380,6 +376,7 @@ class W_StructureInstanceAutoFree(W_StructureInstance):
     def __init__(self, space, shape):
         W_StructureInstance.__init__(self, space, shape, 0)
 
+    @rgc.must_be_light_finalizer
     def __del__(self):
         if self.ll_buffer:
             self._free()
@@ -389,7 +386,6 @@ W_StructureInstanceAutoFree.typedef = TypeDef(
     __repr__    = interp2app(W_StructureInstance.descr_repr),
     __getattr__ = interp2app(W_StructureInstance.getattr),
     __setattr__ = interp2app(W_StructureInstance.setattr),
-    __buffer__  = interp2app(W_StructureInstance.descr_buffer),
     buffer      = GetSetProperty(W_StructureInstance.getbuffer),
     shape       = interp_attrproperty('shape', W_StructureInstance),
     byptr       = interp2app(W_StructureInstance.byptr),
