@@ -15,6 +15,7 @@ from pypy.interpreter.gateway import (
     interp2app, interpindirect2app, unwrap_spec)
 from pypy.interpreter.typedef import (
     GetSetProperty, TypeDef, make_weakref_descr)
+from pypy.interpreter.generator import GeneratorIterator
 from pypy.module._file.interp_file import W_File
 from pypy.objspace.std.floatobject import W_FloatObject
 
@@ -227,8 +228,11 @@ class W_ArrayBase(W_Root):
         Convert the array to an array of machine values and return the string
         representation.
         """
+        size = self.len
+        if size == 0:
+            return space.wrap('')
         cbuf = self._charbuf_start()
-        s = rffi.charpsize2str(cbuf, self.len * self.itemsize)
+        s = rffi.charpsize2str(cbuf, size * self.itemsize)
         self._charbuf_stop()
         return self.space.wrap(s)
 
@@ -630,6 +634,10 @@ class ArrayBuffer(Buffer):
 def make_array(mytype):
     W_ArrayBase = globals()['W_ArrayBase']
 
+    unpack_driver = jit.JitDriver(name='unpack_array',
+                                  greens=['tp'],
+                                  reds=['self', 'w_iterator'])
+
     class W_Array(W_ArrayBase):
         itemsize = mytype.bytes
         typecode = mytype.typecode
@@ -674,6 +682,10 @@ def make_array(mytype):
                 return rffi.cast(mytype.itemtype, item)
             #
             # "regular" case: it fits in an rpython integer (lltype.Signed)
+            # or it is a float
+            return self.item_from_int_or_float(item)
+
+        def item_from_int_or_float(self, item):
             result = rffi.cast(mytype.itemtype, item)
             if mytype.canoverflow:
                 if rffi.cast(lltype.Signed, result) != item:
@@ -686,8 +698,8 @@ def make_array(mytype):
                                % mytype.bytes)
                     if not mytype.signed:
                         msg = 'un' + msg      # 'signed' => 'unsigned'
-                    raise OperationError(space.w_OverflowError,
-                                         space.wrap(msg))
+                    raise OperationError(self.space.w_OverflowError,
+                                         self.space.wrap(msg))
             return result
 
         def __del__(self):
@@ -734,27 +746,65 @@ def make_array(mytype):
         def fromsequence(self, w_seq):
             space = self.space
             oldlen = self.len
-            try:
-                new = space.len_w(w_seq)
-                self.setlen(self.len + new)
-            except OperationError:
-                pass
+            newlen = oldlen
 
-            i = 0
-            try:
-                if mytype.typecode == 'u':
-                    myiter = space.unpackiterable
-                else:
-                    myiter = space.listview
-                for w_i in myiter(w_seq):
-                    if oldlen + i >= self.len:
-                        self.setlen(oldlen + i + 1)
-                    self.buffer[oldlen + i] = self.item_w(w_i)
-                    i += 1
-            except OperationError:
-                self.setlen(oldlen + i)
-                raise
-            self.setlen(oldlen + i)
+            # optimized case for arrays of integers or floats
+            if mytype.unwrap == 'int_w':
+                lst = space.listview_int(w_seq)
+            elif mytype.unwrap == 'float_w':
+                lst = space.listview_float(w_seq)
+            else:
+                lst = None
+            if lst is not None:
+                self.setlen(oldlen + len(lst))
+                try:
+                    buf = self.buffer
+                    for num in lst:
+                        buf[newlen] = self.item_from_int_or_float(num)
+                        newlen += 1
+                except OperationError:
+                    self.setlen(newlen)
+                    raise
+                return
+
+            # this is the common case: w_seq is a list or a tuple
+            lst_w = space.listview_no_unpack(w_seq)
+            if lst_w is not None:
+                self.setlen(oldlen + len(lst_w))
+                buf = self.buffer
+                try:
+                    for w_num in lst_w:
+                        # note: self.item_w() might invoke arbitrary code.
+                        # In case it resizes the same array, then strange
+                        # things may happen, but as we don't reload 'buf'
+                        # we know that one is big enough for all items
+                        # (so at least we avoid crashes)
+                        buf[newlen] = self.item_w(w_num)
+                        newlen += 1
+                except OperationError:
+                    if buf == self.buffer:
+                        self.setlen(newlen)
+                    raise
+                return
+
+            self._fromiterable(w_seq)
+
+        def _fromiterable(self, w_seq):
+            # a more careful case if w_seq happens to be a very large
+            # iterable: don't copy the items into some intermediate list
+            w_iterator = self.space.iter(w_seq)
+            tp = self.space.type(w_iterator)
+            while True:
+                unpack_driver.jit_merge_point(tp=tp, self=self,
+                                              w_iterator=w_iterator)
+                space = self.space
+                try:
+                    w_item = space.next(w_iterator)
+                except OperationError, e:
+                    if not e.match(space, space.w_StopIteration):
+                        raise
+                    break  # done
+                self.descr_append(space, w_item)
 
         def extend(self, w_iterable, accept_different_array=False):
             space = self.space
@@ -797,8 +847,9 @@ def make_array(mytype):
 
         def descr_append(self, space, w_x):
             x = self.item_w(w_x)
-            self.setlen(self.len + 1)
-            self.buffer[self.len - 1] = x
+            index = self.len
+            self.setlen(index + 1)
+            self.buffer[index] = x
 
         # List interface
         def descr_count(self, space, w_val):
