@@ -150,14 +150,29 @@ def PyErr_SetFromErrnoWithFilename(space, w_type, llfilename):
     this is used to define the filename attribute of the exception instance.
     Return value: always NULL."""
     # XXX Doesn't actually do anything with PyErr_CheckSignals.
-    errno = get_errno()
-    msg = os.strerror(errno)
     if llfilename:
         w_filename = rffi.charp2str(llfilename)
+        filename = space.wrap(w_filename)
+    else:
+        filename = space.w_None
+
+    PyErr_SetFromErrnoWithFilenameObject(space, w_type, filename)
+
+@cpython_api([PyObject, PyObject], PyObject)
+def PyErr_SetFromErrnoWithFilenameObject(space, w_type, w_value):
+    """Similar to PyErr_SetFromErrno(), with the additional behavior that if
+    w_value is not NULL, it is passed to the constructor of type as a
+    third parameter.  In the case of exceptions such as IOError and OSError,
+    this is used to define the filename attribute of the exception instance.
+    Return value: always NULL."""
+    # XXX Doesn't actually do anything with PyErr_CheckSignals.
+    errno = get_errno()
+    msg = os.strerror(errno)
+    if w_value:
         w_error = space.call_function(w_type,
                                       space.wrap(errno),
                                       space.wrap(msg),
-                                      space.wrap(w_filename))
+                                      w_value)
     else:
         w_error = space.call_function(w_type,
                                       space.wrap(errno),
