@@ -26,9 +26,14 @@ for t in types:
 del types
 
 types = ['Generic', 'Number', 'Integer', 'SignedInteger', 'UnsignedInteger',
-         'Inexact', 'Floating', 'ComplexFloating', 'Character']
+         'Inexact', 'Floating', 'ComplexFloating', 'Flexible', 'Character']
 for t in types:
     globals()[t.lower()] = typeinfo[t]
 
 True_ = bool_(True)
 False_ = bool_(False)
+
+def ones(*args, **kwargs):
+    a = zeros(*args, **kwargs)
+    a.fill(1)
+    return a
