@@ -7,6 +7,7 @@ from rpython.rlib.rarithmetic import LONG_BIT, maxint
 from rpython.tool.sourcetools import func_with_new_name
 from pypy.module.micronumpy import boxes, descriptor, loop, constants as NPY
 from pypy.module.micronumpy.base import convert_to_array, W_NDimArray
+from pypy.module.micronumpy.ctors import numpify
 from pypy.module.micronumpy.strides import shape_agreement
 
 
@@ -17,6 +18,13 @@ def done_if_true(dtype, val):
 def done_if_false(dtype, val):
     return not dtype.itemtype.bool(val)
 
+def _get_dtype(space, w_npyobj):
+    if isinstance(w_npyobj, boxes.W_GenericBox):
+        return w_npyobj.get_dtype(space)
+    else:
+        assert isinstance(w_npyobj, W_NDimArray)
+        return w_npyobj.get_dtype()
+
 
 class W_Ufunc(W_Root):
     _immutable_fields_ = [
@@ -304,8 +312,8 @@ class W_Ufunc1(W_Ufunc):
             out = args_w[1]
             if space.is_w(out, space.w_None):
                 out = None
-        w_obj = convert_to_array(space, w_obj)
-        dtype = w_obj.get_dtype()
+        w_obj = numpify(space, w_obj)
+        dtype = _get_dtype(space, w_obj)
         if dtype.is_flexible():
             raise OperationError(space.w_TypeError,
                       space.wrap('Not implemented for this type'))
@@ -315,7 +323,7 @@ class W_Ufunc1(W_Ufunc):
             raise oefmt(space.w_TypeError,
                 "ufunc %s not supported for the input type", self.name)
         calc_dtype = find_unaryop_result_dtype(space,
-                                  w_obj.get_dtype(),
+                                  dtype,
                                   promote_to_float=self.promote_to_float,
                                   promote_bools=self.promote_bools)
         if out is not None:
@@ -345,6 +353,7 @@ class W_Ufunc1(W_Ufunc):
             else:
                 out.fill(space, w_val)
             return out
+        assert isinstance(w_obj, W_NDimArray)
         shape = shape_agreement(space, w_obj.get_shape(), out,
                                 broadcast_down=False)
         return loop.call1(space, shape, self.func, calc_dtype, res_dtype,
@@ -385,10 +394,10 @@ class W_Ufunc2(W_Ufunc):
         else:
             [w_lhs, w_rhs] = args_w
             w_out = None
-        w_lhs = convert_to_array(space, w_lhs)
-        w_rhs = convert_to_array(space, w_rhs)
-        w_ldtype = w_lhs.get_dtype()
-        w_rdtype = w_rhs.get_dtype()
+        w_lhs = numpify(space, w_lhs)
+        w_rhs = numpify(space, w_rhs)
+        w_ldtype = _get_dtype(space, w_lhs)
+        w_rdtype = _get_dtype(space, w_rhs)
         if w_ldtype.is_str() and w_rdtype.is_str() and \
                 self.comparison_func:
             pass
@@ -451,6 +460,12 @@ class W_Ufunc2(W_Ufunc):
             else:
                 out = arr
             return out
+        if isinstance(w_lhs, boxes.W_GenericBox):
+            w_lhs = W_NDimArray.from_scalar(space, w_lhs)
+        assert isinstance(w_lhs, W_NDimArray)
+        if isinstance(w_rhs, boxes.W_GenericBox):
+            w_rhs = W_NDimArray.from_scalar(space, w_rhs)
+        assert isinstance(w_rhs, W_NDimArray)
         new_shape = shape_agreement(space, w_lhs.get_shape(), w_rhs)
         new_shape = shape_agreement(space, new_shape, out, broadcast_down=False)
         return loop.call2(space, new_shape, self.func, calc_dtype,
