@@ -4,7 +4,7 @@ import errno
 
 from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.gateway import interp2app, unwrap_spec
-from pypy.interpreter.error import OperationError, operationerrfmt, exception_from_errno
+from pypy.interpreter.error import OperationError, exception_from_errno, oefmt
 from pypy.interpreter.typedef import TypeDef, GetSetProperty
 from rpython.rtyper.lltypesystem import lltype, rffi
 from rpython.rtyper.tool import rffi_platform
@@ -78,9 +78,8 @@ class W_Epoll(W_Root):
         if sizehint == -1:
             sizehint = FD_SETSIZE - 1
         elif sizehint < 0:
-            raise operationerrfmt(space.w_ValueError,
-                "sizehint must be greater than zero, got %d", sizehint
-            )
+            raise oefmt(space.w_ValueError,
+                        "sizehint must be greater than zero, got %d", sizehint)
         epfd = epoll_create(sizehint)
         if epfd < 0:
             raise exception_from_errno(space, space.w_IOError)
@@ -155,9 +154,8 @@ class W_Epoll(W_Root):
         if maxevents == -1:
             maxevents = FD_SETSIZE - 1
         elif maxevents < 1:
-            raise operationerrfmt(space.w_ValueError,
-                "maxevents must be greater than 0, not %d", maxevents
-            )
+            raise oefmt(space.w_ValueError,
+                        "maxevents must be greater than 0, not %d", maxevents)
 
         with lltype.scoped_alloc(rffi.CArray(epoll_event), maxevents) as evs:
             nfds = epoll_wait(self.epfd, evs, maxevents, int(timeout))
