@@ -2,12 +2,11 @@
 Tests for the struct module implemented at interp-level in pypy/module/struct.
 """
 
-import py
 from rpython.rlib.rstruct.nativefmttable import native_is_bigendian
 
 
 class AppTestStruct(object):
-    spaceconfig = dict(usemodules=['struct'])
+    spaceconfig = dict(usemodules=['struct', 'array'])
 
     def setup_class(cls):
         """
@@ -20,13 +19,15 @@ class AppTestStruct(object):
         """)
         cls.w_native_is_bigendian = cls.space.wrap(native_is_bigendian)
 
-
     def test_error(self):
         """
         struct.error should be an exception class.
         """
         assert issubclass(self.struct.error, Exception)
-
+        assert self.struct.error.__mro__ == (self.struct.error, Exception,
+                                             BaseException, object)
+        assert self.struct.error.__name__ == "error"
+        assert self.struct.error.__module__ == "struct"
 
     def test_calcsize_standard(self):
         """
@@ -53,14 +54,12 @@ class AppTestStruct(object):
         # test with some repetitions and multiple format characters
         assert calcsize('=bQ3i') == 1 + 8 + 3*4
 
-
     def test_index(self):
         class X(object):
             def __index__(self):
                 return 3
         assert self.struct.unpack("i", self.struct.pack("i", X()))[0] == 3
 
-
     def test_deprecation_warning(self):
         import warnings
         for code in 'b', 'B', 'h', 'H', 'i', 'I', 'l', 'L', 'q', 'Q':
@@ -71,7 +70,6 @@ class AppTestStruct(object):
             assert str(w[0].message) == "integer argument expected, got non-integer"
             assert w[0].category is DeprecationWarning
 
-
     def test_pack_standard_little(self):
         """
         Check packing with the '<' format specifier.
@@ -85,7 +83,6 @@ class AppTestStruct(object):
         assert pack("<q", -0x41B2B3B4B5B6B7B8) == 'HHIJKLM\xbe'
         assert pack("<Q", 0x8142434445464748) == 'HGFEDCB\x81'
 
-
     def test_unpack_standard_little(self):
         """
         Check unpacking with the '<' format specifier.
@@ -99,7 +96,6 @@ class AppTestStruct(object):
         assert unpack("<q", 'HHIJKLM\xbe') == (-0x41B2B3B4B5B6B7B8,)
         assert unpack("<Q", 'HGFEDCB\x81') == (0x8142434445464748,)
 
-
     def test_pack_standard_big(self):
         """
         Check packing with the '>' format specifier.
@@ -113,7 +109,6 @@ class AppTestStruct(object):
         assert pack(">q", -0x41B2B3B4B5B6B7B8) == '\xbeMLKJIHH'
         assert pack(">Q", 0x8142434445464748) == '\x81BCDEFGH'
 
-
     def test_unpack_standard_big(self):
         """
         Check unpacking with the '>' format specifier.
@@ -127,7 +122,6 @@ class AppTestStruct(object):
         assert unpack(">q", '\xbeMLKJIHH') == (-0x41B2B3B4B5B6B7B8,)
         assert unpack(">Q", '\x81BCDEFGH') == (0x8142434445464748,)
 
-
     def test_calcsize_native(self):
         """
         Check that the size of the various format characters is reasonable.
@@ -157,7 +151,6 @@ class AppTestStruct(object):
         assert calcsize('ibb') == calcsize('i') + 2 * calcsize('b')
         assert calcsize('ih') == calcsize('i') + calcsize('h')
 
-
     def test_pack_native(self):
         """
         Check packing with the native format.
@@ -175,7 +168,6 @@ class AppTestStruct(object):
             assert res[sizeofi:] == '\x05' + '\x00' * (sizeofi-1)
         assert pack("q", -1) == '\xff' * calcsize("q")
 
-
     def test_unpack_native(self):
         """
         Check unpacking with the native format.
@@ -186,7 +178,6 @@ class AppTestStruct(object):
         assert unpack("bi", pack("bi", -2, 5)) == (-2, 5)
         assert unpack("q", '\xff' * calcsize("q")) == (-1,)
 
-
     def test_string_format(self):
         """
         Check the 's' format character.
@@ -201,7 +192,6 @@ class AppTestStruct(object):
         assert unpack("5s3s", "worldspa") == ("world", "spa")
         assert unpack("0s", "") == ("",)
 
-
     def test_pascal_format(self):
         """
         Check the 'p' format character.
@@ -221,7 +211,6 @@ class AppTestStruct(object):
         assert unpack("1p", "\x03") == ("",)
         assert unpack("300p", longpacked300) == (longstring[:255],)
 
-
     def test_char_format(self):
         """
         Check the 'c' format character.
@@ -233,7 +222,6 @@ class AppTestStruct(object):
         assert unpack("c", "?") == ("?",)
         assert unpack("5c", "a\xc0\x00\n-") == ("a", "\xc0", "\x00", "\n", "-")
 
-
     def test_pad_format(self):
         """
         Check the 'x' format character.
@@ -245,7 +233,6 @@ class AppTestStruct(object):
         assert unpack("x", "?") == ()
         assert unpack("5x", "hello") == ()
 
-
     def test_native_floats(self):
         """
         Check the 'd' and 'f' format characters on native packing.
@@ -262,7 +249,6 @@ class AppTestStruct(object):
         assert res != 12.34                      # precision lost
         assert abs(res - 12.34) < 1E-6
 
-
     def test_standard_floats(self):
         """
         Check the 'd' and 'f' format characters on standard packing.
@@ -281,7 +267,6 @@ class AppTestStruct(object):
 
     def test_bool(self):
         pack = self.struct.pack
-        unpack = self.struct.unpack
         assert pack("!?", True) == '\x01'
         assert pack(">?", True) == '\x01'
         assert pack("!?", False) == '\x00'
@@ -344,15 +329,12 @@ class AppTestStruct(object):
         raises(error, pack, "b", 150)   # argument out of range
         # XXX the accepted ranges still differs between PyPy and CPython
 
-
     def test_overflow_error(self):
         """
         Check OverflowError cases.
         """
         import sys
         calcsize = self.struct.calcsize
-        pack = self.struct.pack
-        unpack = self.struct.unpack
         someerror = (OverflowError, self.struct.error)
         raises(someerror, calcsize, "%dc" % (sys.maxint+1,))
         raises(someerror, calcsize, "999999999999999999999999999c")
@@ -361,7 +343,6 @@ class AppTestStruct(object):
         raises(someerror, calcsize, "c%dc" % (sys.maxint,))
         raises(someerror, calcsize, "%dci" % (sys.maxint,))
 
-
     def test_unicode(self):
         """
         A PyPy extension: accepts the 'u' format character in native mode,
@@ -375,7 +356,6 @@ class AppTestStruct(object):
         assert data == str(buffer(u'XYZ'))
         assert self.struct.unpack("uuu", data) == (u'X', u'Y', u'Z')
 
-
     def test_unpack_buffer(self):
         """
         Buffer objects can be passed to struct.unpack().
@@ -384,6 +364,54 @@ class AppTestStruct(object):
         assert self.struct.unpack("ii", b) == (62, 12)
         raises(self.struct.error, self.struct.unpack, "i", b)
 
+    def test_pack_unpack_buffer(self):
+        import array
+        b = array.array('c', '\x00' * 19)
+        sz = self.struct.calcsize("ii")
+        for offset in [2, -17]:
+            self.struct.pack_into("ii", b, offset, 17, 42)
+            assert str(buffer(b)) == ('\x00' * 2 +
+                                      self.struct.pack("ii", 17, 42) +
+                                      '\x00' * (19-sz-2))
+        exc = raises(TypeError, self.struct.pack_into, "ii", buffer(b), 0, 17, 42)
+        assert str(exc.value) == "buffer is read-only"
+        exc = raises(TypeError, self.struct.pack_into, "ii", 'test', 0, 17, 42)
+        assert str(exc.value) == "Cannot use string as modifiable buffer"
+        exc = raises(self.struct.error, self.struct.pack_into, "ii", b[0:1], 0, 17, 42)
+        assert str(exc.value) == "pack_into requires a buffer of at least 8 bytes"
+
+        assert self.struct.unpack_from("ii", b, 2) == (17, 42)
+        assert self.struct.unpack_from("ii", b, -17) == (17, 42)
+        assert self.struct.unpack_from("ii", buffer(b, 2)) == (17, 42)
+        assert self.struct.unpack_from("ii", buffer(b), 2) == (17, 42)
+        assert self.struct.unpack_from("ii", memoryview(buffer(b)), 2) == (17, 42)
+        exc = raises(TypeError, self.struct.unpack_from, "ii", 123)
+        assert 'must be string or buffer, not int' in str(exc.value)
+        exc = raises(self.struct.error, self.struct.unpack_from, "ii", None)
+        assert str(exc.value) == "unpack_from requires a buffer argument"
+        exc = raises(self.struct.error, self.struct.unpack_from, "ii", '')
+        assert str(exc.value) == "unpack_from requires a buffer of at least 8 bytes"
+        exc = raises(self.struct.error, self.struct.unpack_from, "ii", memoryview(''))
+        assert str(exc.value) == "unpack_from requires a buffer of at least 8 bytes"
+
+    def test___float__(self):
+        class MyFloat(object):
+            def __init__(self, x):
+                self.x = x
+            def __float__(self):
+                return self.x
+
+        obj = MyFloat(42.3)
+        data = self.struct.pack('d', obj)
+        obj2, = self.struct.unpack('d', data)
+        assert type(obj2) is float
+        assert obj2 == 42.3
+
+    def test_struct_object(self):
+        s = self.struct.Struct('i')
+        assert s.unpack(s.pack(42)) == (42,)
+        assert s.unpack_from(memoryview(s.pack(42))) == (42,)
+
 
 class AppTestStructBuffer(object):
     spaceconfig = dict(usemodules=['struct', '__pypy__'])
