@@ -1,12 +1,14 @@
 
 from pypy.interpreter.baseobjspace import W_Root
-from pypy.interpreter.error import OperationError, operationerrfmt
+from pypy.interpreter.error import OperationError, oefmt
 from pypy.interpreter.gateway import interp2app, unwrap_spec
 from pypy.interpreter.typedef import TypeDef, GetSetProperty
 from pypy.interpreter.module import Module
 from pypy.module.imp import importing
+from pypy.module.zlib.interp_zlib import zlib_error
 from rpython.rlib.unroll import unrolling_iterable
 from rpython.rlib.rzipfile import RZipFile, BadZipfile
+from rpython.rlib.rzlib import RZlibError
 import os
 import stat
 
@@ -54,6 +56,8 @@ class W_ZipCache(W_Root):
         w = space.wrap
         w_d = space.newdict()
         for key, info in w_zipimporter.zip_file.NameToInfo.iteritems():
+            if ZIPSEP != os.path.sep:
+                key = key.replace(ZIPSEP, os.path.sep)
             space.setitem(w_d, w(key), space.newtuple([
                 w(info.filename), w(info.compress_type), w(info.compress_size),
                 w(info.file_size), w(info.file_offset), w(info.dostime),
@@ -252,6 +256,10 @@ class W_ZipImporter(W_Root):
                 buf = self.zip_file.read(fname)
             except (KeyError, OSError, BadZipfile):
                 pass
+            except RZlibError, e:
+                # in this case, CPython raises the direct exception coming
+                # from the zlib module: let's to the same
+                raise zlib_error(space, e.msg)
             else:
                 if is_package:
                     pkgpath = (self.filename + os.path.sep +
@@ -283,6 +291,10 @@ class W_ZipImporter(W_Root):
             return w(data)
         except (KeyError, OSError, BadZipfile):
             raise OperationError(space.w_IOError, space.wrap("Error reading file"))
+        except RZlibError, e:
+            # in this case, CPython raises the direct exception coming
+            # from the zlib module: let's to the same
+            raise zlib_error(space, e.msg)
 
     @unwrap_spec(fullname=str)
     def get_code(self, space, fullname):
@@ -304,8 +316,9 @@ class W_ZipImporter(W_Root):
                     code_w = importing.parse_source_module(
                         space, co_filename, source)
                 return space.wrap(code_w)
-        raise operationerrfmt(get_error(space),
-            "Cannot find source or code for %s in %s", filename, self.name)
+        raise oefmt(get_error(space),
+                    "Cannot find source or code for %s in %s",
+                    filename, self.name)
 
     @unwrap_spec(fullname=str)
     def get_source(self, space, fullname):
@@ -320,8 +333,8 @@ class W_ZipImporter(W_Root):
                     found = True
         if found:
             return space.w_None
-        raise operationerrfmt(get_error(space),
-            "Cannot find source for %s in %s", filename, self.name)
+        raise oefmt(get_error(space),
+                    "Cannot find source for %s in %s", filename, self.name)
 
     @unwrap_spec(fullname=str)
     def get_filename(self, space, fullname):
@@ -330,8 +343,8 @@ class W_ZipImporter(W_Root):
             if self.have_modulefile(space, filename + ext):
                 return space.wrap(self.filename + os.path.sep +
                                   self.corr_zname(filename + ext))
-        raise operationerrfmt(get_error(space),
-            "Cannot find module %s in %s", filename, self.name)
+        raise oefmt(get_error(space),
+                    "Cannot find module %s in %s", filename, self.name)
 
     @unwrap_spec(fullname=str)
     def is_package(self, space, fullname):
@@ -339,8 +352,8 @@ class W_ZipImporter(W_Root):
         for _, is_package, ext in ENUMERATE_EXTS:
             if self.have_modulefile(space, filename + ext):
                 return space.wrap(is_package)
-        raise operationerrfmt(get_error(space),
-            "Cannot find module %s in %s", filename, self.name)
+        raise oefmt(get_error(space),
+                    "Cannot find module %s in %s", filename, self.name)
 
     def getarchive(self, space):
         space = self.space
@@ -360,27 +373,30 @@ def descr_new_zipimporter(space, w_type, name):
         try:
             s = os.stat(filename)
         except OSError:
-            raise operationerrfmt(get_error(space),
-                "Cannot find name %s", filename)
+            raise oefmt(get_error(space), "Cannot find name %s", filename)
         if not stat.S_ISDIR(s.st_mode):
             ok = True
             break
     if not ok:
-        raise operationerrfmt(get_error(space),
-            "Did not find %s to be a valid zippath", name)
+        raise oefmt(get_error(space), "Did not find %s to be a valid zippath",
+                    name)
     try:
         w_result = zip_cache.get(filename)
         if w_result is None:
-            raise operationerrfmt(get_error(space),
-                "Cannot import %s from zipfile, recursion detected or"
-                "already tried and failed", name)
+            raise oefmt(get_error(space),
+                        "Cannot import %s from zipfile, recursion detected or"
+                        "already tried and failed", name)
     except KeyError:
         zip_cache.cache[filename] = None
     try:
         zip_file = RZipFile(filename, 'r')
     except (BadZipfile, OSError):
-        raise operationerrfmt(get_error(space),
-            "%s seems not to be a zipfile", filename)
+        raise oefmt(get_error(space), "%s seems not to be a zipfile", filename)
+    except RZlibError, e:
+        # in this case, CPython raises the direct exception coming
+        # from the zlib module: let's to the same
+        raise zlib_error(space, e.msg)
+
     prefix = name[len(filename):]
     if prefix.startswith(os.path.sep) or prefix.startswith(ZIPSEP):
         prefix = prefix[1:]
