@@ -3,6 +3,7 @@ import py
 from zipfile import ZIP_DEFLATED
 
 from pypy.module.zipimport.test.test_zipimport import AppTestZipimport as Base
+BAD_ZIP = str(py.path.local(__file__).dirpath('bad.zip'))
 
 class AppTestZipimportDeflated(Base):
     compression = ZIP_DEFLATED
@@ -16,3 +17,10 @@ class AppTestZipimportDeflated(Base):
         except ImportError:
             py.test.skip("zlib not available, cannot test compressed zipfiles")
         cls.make_class()
+        cls.w_BAD_ZIP = cls.space.wrap(BAD_ZIP)
+
+    def test_zlib_error(self):
+        import zipimport
+        import zlib
+        z = zipimport.zipimporter(self.BAD_ZIP)
+        raises(zlib.error, "z.load_module('mymod')")
