@@ -1,4 +1,3 @@
-
 """
 Tests for the zlib module.
 """
@@ -12,7 +11,7 @@ try:
     from pypy.module.zlib import interp_zlib
 except ImportError:
     import py; py.test.skip("no zlib C library on this machine")
- 
+
 def test_unsigned_to_signed_32bit():
     assert interp_zlib.unsigned_to_signed_32bit(123) == 123
     assert interp_zlib.unsigned_to_signed_32bit(2**31) == -2**31
@@ -34,14 +33,12 @@ class AppTestZlib(object):
         cls.w_expanded = cls.space.wrap(expanded)
         cls.w_compressed = cls.space.wrap(zlib.compress(expanded))
 
-
     def test_error(self):
         """
         zlib.error should be an exception class.
         """
         assert issubclass(self.zlib.error, Exception)
 
-
     def test_crc32(self):
         """
         When called with a string, zlib.crc32 should compute its CRC32 and
@@ -52,7 +49,6 @@ class AppTestZlib(object):
         assert self.zlib.crc32('\0') == -771559539
         assert self.zlib.crc32('hello, world.') == -936931198
 
-
     def test_crc32_start_value(self):
         """
         When called with a string and an integer, zlib.crc32 should compute the
@@ -94,7 +90,6 @@ class AppTestZlib(object):
         assert self.zlib.adler32('hello, world.') == 571147447
         assert self.zlib.adler32('x' * 23) == -2122904887
 
-
     def test_adler32_start_value(self):
         """
         When called with a string and an integer, zlib.adler32 should compute
@@ -114,7 +109,6 @@ class AppTestZlib(object):
         assert self.zlib.adler32('foo', -1) == 45547858
         assert self.zlib.adler32('foo', 99999999999999999999999) == -114818734
 
-
     def test_invalidLevel(self):
         """
         zlib.compressobj should raise ValueError when an out of bounds level is
@@ -123,7 +117,6 @@ class AppTestZlib(object):
         raises(ValueError, self.zlib.compressobj, -2)
         raises(ValueError, self.zlib.compressobj, 10)
 
-
     def test_compression(self):
         """
         zlib.compressobj should return an object which can be used to compress
@@ -131,10 +124,10 @@ class AppTestZlib(object):
         """
         compressor = self.zlib.compressobj()
         bytes = compressor.compress(self.expanded)
+        raises(OverflowError, compressor.flush, 2**31)
         bytes += compressor.flush()
         assert bytes == self.compressed
 
-
     def test_decompression(self):
         """
         zlib.decompressobj should return an object which can be used to
@@ -142,10 +135,10 @@ class AppTestZlib(object):
         """
         decompressor = self.zlib.decompressobj()
         bytes = decompressor.decompress(self.compressed)
+        raises(OverflowError, decompressor.flush, 2**31)
         bytes += decompressor.flush()
         assert bytes == self.expanded
 
-
     def test_compress(self):
         """
         Test the zlib.compress() function.
@@ -153,7 +146,6 @@ class AppTestZlib(object):
         bytes = self.zlib.compress(self.expanded)
         assert bytes == self.compressed
 
-
     def test_decompress(self):
         """
         Test the zlib.decompress() function.
@@ -161,7 +153,6 @@ class AppTestZlib(object):
         bytes = self.zlib.decompress(self.compressed)
         assert bytes == self.expanded
 
-
     def test_decompress_invalid_input(self):
         """
         Try to feed garbage to zlib.decompress().
@@ -169,6 +160,38 @@ class AppTestZlib(object):
         raises(self.zlib.error, self.zlib.decompress, self.compressed[:-2])
         raises(self.zlib.error, self.zlib.decompress, 'foobar')
 
+    def test_bad_arguments(self):
+        import zlib
+        raises(ValueError, zlib.decompressobj().flush, 0)
+        raises(ValueError, zlib.decompressobj().flush, -1)
+        raises(TypeError, zlib.decompressobj().flush, None)
+        raises(OverflowError, zlib.decompressobj().flush, 2**31)
+        raises(ValueError, zlib.decompressobj().decompress, b'abc', -1)
+        raises(TypeError, zlib.decompressobj().decompress, b'abc', None)
+        raises(OverflowError, zlib.decompressobj().decompress, b'abc', 2**31)
+        raises(TypeError, self.zlib.decompress, self.compressed, None)
+        raises(OverflowError, self.zlib.decompress, self.compressed, 2**31)
+
+    def test_empty_flush(self):
+        import zlib
+        co = zlib.compressobj(zlib.Z_BEST_COMPRESSION)
+        assert co.flush()  # Returns a zlib header
+        dco = zlib.decompressobj()
+        assert dco.flush() == b""
+
+    def test_decompress_incomplete_stream(self):
+        import zlib
+        # This is 'foo', deflated
+        x = b'x\x9cK\xcb\xcf\x07\x00\x02\x82\x01E'
+        # For the record
+        assert zlib.decompress(x) == b'foo'
+        raises(zlib.error, zlib.decompress, x[:-5])
+        # Omitting the stream end works with decompressor objects
+        # (see issue #8672).
+        dco = zlib.decompressobj()
+        y = dco.decompress(x[:-5])
+        y += dco.flush()
+        assert y == b'foo'
 
     def test_unused_data(self):
         """
@@ -176,9 +199,11 @@ class AppTestZlib(object):
         It should show up in the unused_data attribute.
         """
         d = self.zlib.decompressobj()
-        s = d.decompress(self.compressed + 'extrastuff')
+        s = d.decompress(self.compressed + 'extrastuff', 0)
         assert s == self.expanded
         assert d.unused_data == 'extrastuff'
+        assert d.flush() == ''
+        assert d.unused_data == 'extrastuff'
         # try again with several decompression steps
         d = self.zlib.decompressobj()
         s1 = d.decompress(self.compressed[:10])
@@ -189,10 +214,9 @@ class AppTestZlib(object):
         assert d.unused_data == 'spam' * 100
         assert s1 + s2 + s3 == self.expanded
         s4 = d.decompress('egg' * 50)
-        assert d.unused_data == 'egg' * 50
+        assert d.unused_data == ('spam' * 100) + ('egg' * 50)
         assert s4 == ''
 
-
     def test_max_length(self):
         """
         Test the max_length argument of the decompress() method
@@ -206,7 +230,6 @@ class AppTestZlib(object):
             data = d.unconsumed_tail
         assert not data
 
-
     def test_buffer(self):
         """
         We should be able to pass buffer objects instead of strings.
@@ -229,3 +252,17 @@ class AppTestZlib(object):
 
         bytes = self.zlib.decompress(buffer(self.compressed))
         assert bytes == self.expanded
+
+    def test_flush_with_freed_input(self):
+        # Issue #16411: decompressor accesses input to last decompress() call
+        # in flush(), even if this object has been freed in the meanwhile.
+        input1 = b'abcdefghijklmnopqrstuvwxyz'
+        input2 = b'QWERTYUIOPASDFGHJKLZXCVBNM'
+        data = self.zlib.compress(input1)
+        dco = self.zlib.decompressobj()
+        dco.decompress(data, 1)
+        del data
+        data = self.zlib.compress(input2)
+        assert dco.flush(1) == input1[1:]
+        assert dco.unused_data == b''
+        assert dco.unconsumed_tail == b''
