@@ -1,9 +1,11 @@
 """Functionality shared between bytes/bytearray/unicode"""
 
 from rpython.rlib import jit
-from rpython.rlib.objectmodel import specialize
+from rpython.rlib.objectmodel import specialize, newlist_hint
 from rpython.rlib.rarithmetic import ovfcheck
-from rpython.rlib.rstring import endswith, replace, rsplit, split, startswith
+from rpython.rlib.rstring import (
+    find, rfind, count, endswith, replace, rsplit, split, startswith)
+from rpython.rlib.buffer import Buffer
 
 from pypy.interpreter.error import OperationError, oefmt
 from pypy.interpreter.gateway import WrappedDefault, unwrap_spec
@@ -28,6 +30,9 @@ class StringMethods(object):
             space, lenself, w_start, w_end, upper_bound=upper_bound)
         return (value, start, end)
 
+    def _multi_chr(self, c):
+        return c
+
     def descr_len(self, space):
         return space.wrap(self._len())
 
@@ -36,17 +41,33 @@ class StringMethods(object):
 
     def descr_contains(self, space, w_sub):
         value = self._val(space)
-        other = self._op_val(space, w_sub)
-        return space.newbool(value.find(other) >= 0)
+        if self._use_rstr_ops(space, w_sub):
+            other = self._op_val(space, w_sub)
+            return space.newbool(value.find(other) >= 0)
+
+        from pypy.objspace.std.bytesobject import W_BytesObject
+        if isinstance(w_sub, W_BytesObject):
+            other = self._op_val(space, w_sub)
+            res = find(value, other, 0, len(value))
+        else:
+            buffer = _get_buffer(space, w_sub)
+            res = find(value, buffer, 0, len(value))
+
+        return space.newbool(res >= 0)
 
     def descr_add(self, space, w_other):
-        try:
-            other = self._op_val(space, w_other)
-        except OperationError as e:
-            if e.match(space, space.w_TypeError):
-                return space.w_NotImplemented
-            raise
-        return self._new(self._val(space) + other)
+        if self._use_rstr_ops(space, w_other):
+            try:
+                other = self._op_val(space, w_other)
+            except OperationError as e:
+                if e.match(space, space.w_TypeError):
+                    return space.w_NotImplemented
+                raise
+            return self._new(self._val(space) + other)
+
+        # Bytearray overrides this method, CPython doesn't support contacting
+        # buffers and strs, and unicodes are always handled above
+        return space.w_NotImplemented
 
     def descr_mul(self, space, w_times):
         try:
@@ -58,7 +79,7 @@ class StringMethods(object):
         if times <= 0:
             return self._empty()
         if self._len() == 1:
-            return self._new(self._val(space)[0] * times)
+            return self._new(self._multi_chr(self._val(space)[0]) * times)
         return self._new(self._val(space) * times)
 
     descr_rmul = descr_mul
@@ -119,7 +140,7 @@ class StringMethods(object):
         d = width - len(value)
         if d > 0:
             offset = d//2 + (d & width & 1)
-            fillchar = fillchar[0]    # annotator hint: it's a single character
+            fillchar = self._multi_chr(fillchar[0])
             centered = offset * fillchar + value + (d - offset) * fillchar
         else:
             centered = value
@@ -128,15 +149,32 @@ class StringMethods(object):
 
     def descr_count(self, space, w_sub, w_start=None, w_end=None):
         value, start, end = self._convert_idx_params(space, w_start, w_end)
-        return space.newint(value.count(self._op_val(space, w_sub), start,
-                                        end))
+
+        if self._use_rstr_ops(space, w_sub):
+            return space.newint(value.count(self._op_val(space, w_sub), start,
+                                            end))
+
+        from pypy.objspace.std.bytearrayobject import W_BytearrayObject
+        from pypy.objspace.std.bytesobject import W_BytesObject
+        if isinstance(w_sub, W_BytearrayObject):
+            res = count(value, w_sub.data, start, end)
+        elif isinstance(w_sub, W_BytesObject):
+            res = count(value, w_sub._value, start, end)
+        else:
+            buffer = _get_buffer(space, w_sub)
+            res = count(value, buffer, start, end)
+
+        return space.wrap(max(res, 0))
 
     def descr_decode(self, space, w_encoding=None, w_errors=None):
         from pypy.objspace.std.unicodeobject import (
             _get_encoding_and_errors, decode_object, unicode_from_string)
         encoding, errors = _get_encoding_and_errors(space, w_encoding,
                                                     w_errors)
-        if encoding is None and errors is None:
+
+        from pypy.objspace.std.bytearrayobject import W_BytearrayObject
+        if (encoding is None and errors is None and
+            not isinstance(self, W_BytearrayObject)):
             return unicode_from_string(space, self)
         return decode_object(space, self, encoding, errors)
 
@@ -153,7 +191,11 @@ class StringMethods(object):
         if not value:
             return self._empty()
 
-        splitted = value.split(self._chr('\t'))
+        if self._use_rstr_ops(space, self):
+            splitted = value.split(self._chr('\t'))
+        else:
+            splitted = split(value, self._chr('\t'))
+
         try:
             ovfcheck(len(splitted) * tabsize)
         except OverflowError:
@@ -161,7 +203,7 @@ class StringMethods(object):
         expanded = oldtoken = splitted.pop(0)
 
         for token in splitted:
-            expanded += self._chr(' ') * self._tabindent(oldtoken,
+            expanded += self._multi_chr(self._chr(' ')) * self._tabindent(oldtoken,
                                                          tabsize) + token
             oldtoken = token
 
@@ -192,30 +234,80 @@ class StringMethods(object):
 
     def descr_find(self, space, w_sub, w_start=None, w_end=None):
         (value, start, end) = self._convert_idx_params(space, w_start, w_end)
-        res = value.find(self._op_val(space, w_sub), start, end)
+
+        if self._use_rstr_ops(space, w_sub):
+            res = value.find(self._op_val(space, w_sub), start, end)
+            return space.wrap(res)
+
+        from pypy.objspace.std.bytearrayobject import W_BytearrayObject
+        from pypy.objspace.std.bytesobject import W_BytesObject
+        if isinstance(w_sub, W_BytearrayObject):
+            res = find(value, w_sub.data, start, end)
+        elif isinstance(w_sub, W_BytesObject):
+            res = find(value, w_sub._value, start, end)
+        else:
+            buffer = _get_buffer(space, w_sub)
+            res = find(value, buffer, start, end)
+
         return space.wrap(res)
 
     def descr_rfind(self, space, w_sub, w_start=None, w_end=None):
         (value, start, end) = self._convert_idx_params(space, w_start, w_end)
-        res = value.rfind(self._op_val(space, w_sub), start, end)
+
+        if self._use_rstr_ops(space, w_sub):
+            res = value.rfind(self._op_val(space, w_sub), start, end)
+            return space.wrap(res)
+
+        from pypy.objspace.std.bytearrayobject import W_BytearrayObject
+        from pypy.objspace.std.bytesobject import W_BytesObject
+        if isinstance(w_sub, W_BytearrayObject):
+            res = rfind(value, w_sub.data, start, end)
+        elif isinstance(w_sub, W_BytesObject):
+            res = rfind(value, w_sub._value, start, end)
+        else:
+            buffer = _get_buffer(space, w_sub)
+            res = rfind(value, buffer, start, end)
+
         return space.wrap(res)
 
     def descr_index(self, space, w_sub, w_start=None, w_end=None):
         (value, start, end) = self._convert_idx_params(space, w_start, w_end)
-        res = value.find(self._op_val(space, w_sub), start, end)
+
+        from pypy.objspace.std.bytearrayobject import W_BytearrayObject
+        from pypy.objspace.std.bytesobject import W_BytesObject
+        if self._use_rstr_ops(space, w_sub):
+            res = value.find(self._op_val(space, w_sub), start, end)
+        elif isinstance(w_sub, W_BytearrayObject):
+            res = find(value, w_sub.data, start, end)
+        elif isinstance(w_sub, W_BytesObject):
+            res = find(value, w_sub._value, start, end)
+        else:
+            buffer = _get_buffer(space, w_sub)
+            res = find(value, buffer, start, end)
+
         if res < 0:
             raise oefmt(space.w_ValueError,
                         "substring not found in string.index")
-
         return space.wrap(res)
 
     def descr_rindex(self, space, w_sub, w_start=None, w_end=None):
         (value, start, end) = self._convert_idx_params(space, w_start, w_end)
-        res = value.rfind(self._op_val(space, w_sub), start, end)
+
+        from pypy.objspace.std.bytearrayobject import W_BytearrayObject
+        from pypy.objspace.std.bytesobject import W_BytesObject
+        if self._use_rstr_ops(space, w_sub):
+            res = value.rfind(self._op_val(space, w_sub), start, end)
+        elif isinstance(w_sub, W_BytearrayObject):
+            res = rfind(value, w_sub.data, start, end)
+        elif isinstance(w_sub, W_BytesObject):
+            res = rfind(value, w_sub._value, start, end)
+        else:
+            buffer = _get_buffer(space, w_sub)
+            res = rfind(value, buffer, start, end)
+
         if res < 0:
             raise oefmt(space.w_ValueError,
                         "substring not found in string.rindex")
-
         return space.wrap(res)
 
     @specialize.arg(2)
@@ -328,6 +420,7 @@ class StringMethods(object):
         value = self._val(space)
 
         prealloc_size = len(value) * (size - 1)
+        unwrapped = newlist_hint(size)
         for i in range(size):
             w_s = list_w[i]
             check_item = self._join_check_item(space, w_s)
@@ -337,13 +430,16 @@ class StringMethods(object):
                             i, w_s)
             elif check_item == 2:
                 return self._join_autoconvert(space, list_w)
-            prealloc_size += len(self._op_val(space, w_s))
+            # XXX Maybe the extra copy here is okay? It was basically going to
+            #     happen anyway, what with being placed into the builder
+            unwrapped.append(self._op_val(space, w_s))
+            prealloc_size += len(unwrapped[i])
 
         sb = self._builder(prealloc_size)
         for i in range(size):
             if value and i != 0:
                 sb.append(value)
-            sb.append(self._op_val(space, list_w[i]))
+            sb.append(unwrapped[i])
         return self._new(sb.build())
 
     def _join_autoconvert(self, space, list_w):
@@ -358,8 +454,8 @@ class StringMethods(object):
                         "ljust() argument 2 must be a single character")
         d = width - len(value)
         if d > 0:
-            fillchar = fillchar[0]    # annotator hint: it's a single character
-            value += d * fillchar
+            fillchar = self._multi_chr(fillchar[0])
+            value = value + fillchar * d
 
         return self._new(value)
 
@@ -372,7 +468,7 @@ class StringMethods(object):
                         "rjust() argument 2 must be a single character")
         d = width - len(value)
         if d > 0:
-            fillchar = fillchar[0]    # annotator hint: it's a single character
+            fillchar = self._multi_chr(fillchar[0])
             value = d * fillchar + value
 
         return self._new(value)
@@ -385,52 +481,76 @@ class StringMethods(object):
         return self._new(builder.build())
 
     def descr_partition(self, space, w_sub):
+        from pypy.objspace.std.bytearrayobject import W_BytearrayObject
         value = self._val(space)
-        sub = self._op_val(space, w_sub)
-        if not sub:
-            raise oefmt(space.w_ValueError, "empty separator")
-        pos = value.find(sub)
+
+        if self._use_rstr_ops(space, w_sub):
+            sub = self._op_val(space, w_sub)
+            sublen = len(sub)
+            if sublen == 0:
+                raise oefmt(space.w_ValueError, "empty separator")
+
+            pos = value.find(sub)
+        else:
+            sub = _get_buffer(space, w_sub)
+            sublen = sub.getlength()
+            if sublen == 0:
+                raise oefmt(space.w_ValueError, "empty separator")
+
+            pos = find(value, sub, 0, len(value))
+            if pos != -1 and isinstance(self, W_BytearrayObject):
+                w_sub = self._new_from_buffer(sub)
+
         if pos == -1:
-            from pypy.objspace.std.bytearrayobject import W_BytearrayObject
             if isinstance(self, W_BytearrayObject):
                 self = self._new(value)
             return space.newtuple([self, self._empty(), self._empty()])
         else:
-            from pypy.objspace.std.bytearrayobject import W_BytearrayObject
-            if isinstance(self, W_BytearrayObject):
-                w_sub = self._new(sub)
             return space.newtuple(
                 [self._sliced(space, value, 0, pos, self), w_sub,
-                 self._sliced(space, value, pos+len(sub), len(value), self)])
+                 self._sliced(space, value, pos + sublen, len(value), self)])
 
     def descr_rpartition(self, space, w_sub):
+        from pypy.objspace.std.bytearrayobject import W_BytearrayObject
         value = self._val(space)
-        sub = self._op_val(space, w_sub)
-        if not sub:
-            raise oefmt(space.w_ValueError, "empty separator")
-        pos = value.rfind(sub)
+
+        if self._use_rstr_ops(space, w_sub):
+            sub = self._op_val(space, w_sub)
+            sublen = len(sub)
+            if sublen == 0:
+                raise oefmt(space.w_ValueError, "empty separator")
+
+            pos = value.rfind(sub)
+        else:
+            sub = _get_buffer(space, w_sub)
+            sublen = sub.getlength()
+            if sublen == 0:
+                raise oefmt(space.w_ValueError, "empty separator")
+
+            pos = rfind(value, sub, 0, len(value))
+            if pos != -1 and isinstance(self, W_BytearrayObject):
+                w_sub = self._new_from_buffer(sub)
+
         if pos == -1:
-            from pypy.objspace.std.bytearrayobject import W_BytearrayObject
             if isinstance(self, W_BytearrayObject):
                 self = self._new(value)
             return space.newtuple([self._empty(), self._empty(), self])
         else:
-            from pypy.objspace.std.bytearrayobject import W_BytearrayObject
-            if isinstance(self, W_BytearrayObject):
-                w_sub = self._new(sub)
             return space.newtuple(
                 [self._sliced(space, value, 0, pos, self), w_sub,
-                 self._sliced(space, value, pos+len(sub), len(value), self)])
+                 self._sliced(space, value, pos + sublen, len(value), self)])
 
     @unwrap_spec(count=int)
     def descr_replace(self, space, w_old, w_new, count=-1):
         input = self._val(space)
+
         sub = self._op_val(space, w_old)
         by = self._op_val(space, w_new)
         try:
             res = replace(input, sub, by, count)
         except OverflowError:
             raise oefmt(space.w_OverflowError, "replace string is too long")
+
         return self._new(res)
 
     @unwrap_spec(maxsplit=int)
@@ -442,10 +562,10 @@ class StringMethods(object):
             return self._newlist_unwrapped(space, res)
 
         by = self._op_val(space, w_sep)
-        bylen = len(by)
-        if bylen == 0:
+        if len(by) == 0:
             raise oefmt(space.w_ValueError, "empty separator")
         res = split(value, by, maxsplit)
+
         return self._newlist_unwrapped(space, res)
 
     @unwrap_spec(maxsplit=int)
@@ -457,10 +577,10 @@ class StringMethods(object):
             return self._newlist_unwrapped(space, res)
 
         by = self._op_val(space, w_sep)
-        bylen = len(by)
-        if bylen == 0:
+        if len(by) == 0:
             raise oefmt(space.w_ValueError, "empty separator")
         res = rsplit(value, by, maxsplit)
+
         return self._newlist_unwrapped(space, res)
 
     @unwrap_spec(keepends=bool)
@@ -616,10 +736,11 @@ class StringMethods(object):
             for char in string:
                 buf.append(table[ord(char)])
         else:
+            # XXX Why not preallocate here too?
             buf = self._builder()
             deletion_table = [False] * 256
-            for c in deletechars:
-                deletion_table[ord(c)] = True
+            for i in range(len(deletechars)):
+                deletion_table[ord(deletechars[i])] = True
             for char in string:
                 if not deletion_table[ord(char)]:
                     buf.append(table[ord(char)])
@@ -636,7 +757,7 @@ class StringMethods(object):
     def descr_zfill(self, space, width):
         selfval = self._val(space)
         if len(selfval) == 0:
-            return self._new(self._chr('0') * width)
+            return self._new(self._multi_chr(self._chr('0')) * width)
         num_zeros = width - len(selfval)
         if num_zeros <= 0:
             # cannot return self, in case it is a subclass of str
@@ -662,3 +783,8 @@ class StringMethods(object):
 @specialize.argtype(0)
 def _descr_getslice_slowpath(selfvalue, start, step, sl):
     return [selfvalue[start + i*step] for i in range(sl)]
+
+def _get_buffer(space, w_obj):
+    return space.buffer_w(w_obj, space.BUF_SIMPLE)
+
+
