@@ -459,7 +459,7 @@ class AppTestAppFloatTest:
 
 class AppTestFloatHex:
     spaceconfig = {
-        "usemodules": ["binascii", "rctime"],
+        'usemodules': ['binascii', 'rctime', 'struct'],
     }
 
     def w_identical(self, x, y):
