@@ -441,6 +441,11 @@ class AppTestListObject(object):
         cls.w_on_arm = cls.space.wrap(platform.machine().startswith('arm'))
         cls.w_runappdirect = cls.space.wrap(cls.runappdirect)
 
+    def test_doc(self):
+        assert list.__doc__ == "list() -> new empty list\nlist(iterable) -> new list initialized from iterable's items"
+        assert list.__new__.__doc__ == "T.__new__(S, ...) -> a new object with type S, a subtype of T"
+        assert list.__init__.__doc__ == "x.__init__(...) initializes x; see help(type(x)) for signature"
+
     def test_getstrategyfromlist_w(self):
         l0 = ["a", "2", "a", True]
         # this raised TypeError on ListStrategies
