@@ -21,11 +21,12 @@ class AppError(Exception):
     def __init__(self, excinfo):
         self.excinfo = excinfo
 
+marker = py.test.mark.applevel
 
 class AppTestFunction(py.test.collect.Function):
     def __init__(self, *args, **kwargs):
         super(AppTestFunction, self).__init__(*args, **kwargs)
-        self.keywords['applevel'] = True
+        self._request.applymarker(marker)
 
     def _prunetraceback(self, traceback):
         return traceback
@@ -116,13 +117,6 @@ class AppClassInstance(py.test.collect.Instance):
 class AppClassCollector(PyPyClassCollector):
     Instance = AppClassInstance
 
-    def _haskeyword(self, keyword):
-        return keyword == 'applevel' or \
-               super(AppClassCollector, self)._haskeyword(keyword)
-
-    def _keywords(self):
-        return super(AppClassCollector, self)._keywords() + ['applevel']
-
     def setup(self):
         super(AppClassCollector, self).setup()
         cls = self.obj
